/*
 * ext/AnimX/AnimX.c
 * 
 * by Noritsugu Nakamura
 * from 1998.05.19
 */

#include <X11/Xlib.h>
#include <AnimX.h>
#include "ruby.h"

static VALUE mAnimX, cWin, cPPM;

typedef struct _Win AnimXwin;
typedef struct _Win *AnimXwinPtr;

typedef struct _PPM AnimXppm;
typedef struct _PPM *AnimXppmPtr;

static void
free_win(Win window)
{
  if (window) {
	DestroyWindow(window);
  }
}

static VALUE
win_s_new(VALUE class, VALUE x, VALUE y, VALUE w, VALUE h)
{
  AnimXwinPtr wptr;

  wptr = CreateWindow(NUM2INT(x), NUM2INT(y),
						NUM2INT(w), NUM2INT(h));
  return Data_Wrap_Struct(class, 0, free_win, wptr);
}

static VALUE
win_geth(VALUE win)
{
  AnimXwinPtr wptr;

  Data_Get_Struct(win, AnimXwin, wptr);
  return INT2NUM(wptr->h);
}

static VALUE
win_getv(VALUE win)
{
  AnimXwinPtr wptr;

  Data_Get_Struct(win, AnimXwin, wptr);
  return INT2NUM(wptr->v);
}

static VALUE
win_setwindowtitle(VALUE win, VALUE str)
{
  AnimXwinPtr wptr;

  Data_Get_Struct(win, AnimXwin, wptr);

  switch (TYPE(str)) {
  case T_STRING:
	SetWindowTitle(wptr, STR2CSTR(str));
	break;
  default:
	rb_raise(rb_eRuntimeError, "not valid value");
	break;
  }
}

static VALUE
win_movewindow(VALUE win, VALUE x, VALUE y)
{
  AnimXwinPtr wptr;

  Data_Get_Struct(win, AnimXwin, wptr);
  MoveWindow(wptr, NUM2INT(x), NUM2INT(y));
  return Qnil;
}

static VALUE
win_resizewindow(VALUE win, VALUE width, VALUE height)
{
  AnimXwinPtr wptr;

  Data_Get_Struct(win, AnimXwin, wptr);
  ResizeWindow(wptr, NUM2INT(width), NUM2INT(height));
  return Qnil;
}

static VALUE
win_setmaxsize(VALUE win, VALUE width, VALUE height)
{
  AnimXwinPtr wptr;

  Data_Get_Struct(win, AnimXwin, wptr);
  SetMaxSize(wptr, NUM2INT(width), NUM2INT(height));
  return Qnil;
}

static VALUE
win_setminsize(VALUE win, VALUE width, VALUE height)
{
  AnimXwinPtr wptr;

  Data_Get_Struct(win, AnimXwin, wptr);
  SetMinSize(wptr, NUM2INT(width), NUM2INT(height));
  return Qnil;
}

static VALUE
win_lockcurrentsize(VALUE win)
{
  AnimXwinPtr wptr;

  Data_Get_Struct(win, AnimXwin, wptr);
  LockCurrentSize(wptr);
  return Qnil;
}

static VALUE
win_setpenwidth(VALUE obj, VALUE width)
{
  SetPenWidth(NUM2INT(width));
  return Qnil;
}

static VALUE
win_setcolor(VALUE obj, VALUE pixel)
{
  SetColor(NUM2INT(pixel));
  return Qnil;
}

static VALUE
win_setbackcolor(VALUE obj, VALUE pixel)
{
  SetBackColor(NUM2INT(pixel));
  return Qnil;
}

static VALUE
win_getcolor(VALUE obj, VALUE colorname)
{
  int pixel;

  switch (TYPE(colorname)) {
  case T_STRING:
	pixel = GetColor(STR2CSTR(colorname));
	break;
  default:
	rb_raise(rb_eRuntimeError, "not valid value");
	break;
  }
  return INT2NUM(pixel);
}

static VALUE
win_getrgb(VALUE obj, VALUE red, VALUE green, VALUE blue)
{
  GetRGB(NUM2INT(red), NUM2INT(green), NUM2INT(blue));
  return Qnil;
}

static VALUE
win_installcolormap(VALUE win)
{
  AnimXwinPtr wptr;

  Data_Get_Struct(win, AnimXwin, wptr);
  InstallColormap(wptr);
  return Qnil;
}

static VALUE
win_drawpoint(VALUE win, VALUE x, VALUE y)
{
  AnimXwinPtr wptr;

  Data_Get_Struct(win, AnimXwin, wptr);
  DrawPoint(wptr, NUM2INT(x), NUM2INT(y));
  return Qnil;
}

static VALUE
win_drawline(VALUE win, VALUE x, VALUE y, VALUE ex, VALUE ey)
{
  AnimXwinPtr wptr;

  Data_Get_Struct(win, AnimXwin, wptr);
  DrawLine(wptr, NUM2INT(x), NUM2INT(y), NUM2INT(ex), NUM2INT(ey));
  return Qnil;
}

static VALUE
win_drawrect(VALUE win, VALUE x, VALUE y, VALUE width, VALUE height)
{
  AnimXwinPtr wptr;

  Data_Get_Struct(win, AnimXwin, wptr);
  DrawRect(wptr, NUM2INT(x), NUM2INT(y), NUM2INT(width), NUM2INT(height));
  return Qnil;
}

static VALUE
win_drawcirc(VALUE win, VALUE x, VALUE y, VALUE width, VALUE height)
{
  AnimXwinPtr wptr;

  Data_Get_Struct(win, AnimXwin, wptr);
  DrawCirc(wptr, NUM2INT(x), NUM2INT(y), NUM2INT(width), NUM2INT(height));
  return Qnil;
}

static VALUE
win_fillrect(VALUE win, VALUE x, VALUE y, VALUE width, VALUE height)
{
  AnimXwinPtr wptr;

  Data_Get_Struct(win, AnimXwin, wptr);
  FillRect(wptr, NUM2INT(x), NUM2INT(y), NUM2INT(width), NUM2INT(height));
  return Qnil;
}

static VALUE
win_fillcirc(VALUE win, VALUE x, VALUE y, VALUE width, VALUE height)
{
  AnimXwinPtr wptr;

  Data_Get_Struct(win, AnimXwin, wptr);
  FillCirc(wptr, NUM2INT(x), NUM2INT(y), NUM2INT(width), NUM2INT(height));
  return Qnil;
}

static VALUE
win_drawlines(VALUE win, VALUE points)
{
  AnimXwinPtr wptr;
  int i, len;
  XPoint *XPointPtr;

  Data_Get_Struct(win, AnimXwin, wptr);
  len = RARRAY(points)->len / 2;
  XPointPtr = ALLOCA_N(XPoint, len);
  for (i = 0; i < len; i++) {
	XPointPtr[i].x = RARRAY(points)->ptr[i * 2];
	XPointPtr[i].x = RARRAY(points)->ptr[i * 2 + 1];
  }

  DrawLines(wptr, XPointPtr, len);
  return Qnil;
}

static VALUE
win_filllines(VALUE win, VALUE points)
{
  AnimXwinPtr wptr;
  int i, len;
  XPoint *XPointPtr;

  Data_Get_Struct(win, AnimXwin, wptr);
  len = RARRAY(points)->len / 2;
  XPointPtr = ALLOCA_N(XPoint, len);
  for (i = 0; i < len; i++) {
	XPointPtr[i].x = RARRAY(points)->ptr[i * 2];
	XPointPtr[i].x = RARRAY(points)->ptr[i * 2 + 1];
  }

  FillLines(wptr, XPointPtr, len);
  return Qnil;
}

static VALUE
win_drawstring(VALUE win, VALUE x, VALUE y, VALUE string)
{
  AnimXwinPtr wptr;

  Data_Get_Struct(win, AnimXwin, wptr);
  switch (TYPE(string)) {
  case T_STRING:
	DrawString(wptr, NUM2INT(x), NUM2INT(y), STR2CSTR(string));
	break;
  default:
	rb_raise(rb_eRuntimeError, "not valid value");
	break;
  }
  return Qnil;
}

static VALUE
win_drawkanji(VALUE win, VALUE x, VALUE y, VALUE string)
{
  AnimXwinPtr wptr;

  Data_Get_Struct(win, AnimXwin, wptr);
  switch (TYPE(string)) {
  case T_STRING:
	DrawKanji(wptr, NUM2INT(x), NUM2INT(y), STR2CSTR(string));
	break;
  default:
	rb_raise(rb_eRuntimeError, "not valid value");
	break;
  }
  return Qnil;
}

static VALUE
win_drawimagestring(VALUE win, VALUE x, VALUE y, VALUE string)
{
  AnimXwinPtr wptr;

  Data_Get_Struct(win, AnimXwin, wptr);
  switch (TYPE(string)) {
  case T_STRING:
	DrawImageString(wptr, NUM2INT(x), NUM2INT(y), STR2CSTR(string));
	break;
  default:
	rb_raise(rb_eRuntimeError, "not valid value");
	break;
  }
  return Qnil;
}

static VALUE
win_drawimagekanji(VALUE win, VALUE x, VALUE y, VALUE string)
{
  AnimXwinPtr wptr;

  Data_Get_Struct(win, AnimXwin, wptr);
  switch (TYPE(string)) {
  case T_STRING:
	DrawImageKanji(wptr, NUM2INT(x), NUM2INT(y), STR2CSTR(string));
	break;
  default:
	rb_raise(rb_eRuntimeError, "not valid value");
	break;
  }
  return Qnil;
}

static VALUE
win_loadppm(VALUE class, VALUE filename, VALUE flag)
{
  AnimXppmPtr pptr;

  switch (TYPE(filename)) {
  case T_STRING:
	pptr = LoadPPM(STR2CSTR(filename), NUM2INT(flag));
	break;
  default:
	rb_raise(rb_eRuntimeError, "not valid value");
	break;
  }
  return Data_Wrap_Struct(cPPM, 0, 0, pptr);
}

static VALUE
win_makeemptyppm(VALUE class, VALUE width, VALUE height)
{
  AnimXppmPtr pptr;

  pptr = MakeEmptyPPM(NUM2INT(width), NUM2INT(height));
  return Data_Wrap_Struct(class, 0, 0, pptr);
}

static VALUE
win_savewindow(VALUE win)
{
  AnimXwinPtr wptr;

  Data_Get_Struct(win, AnimXwin, wptr);
  SaveWindow(wptr);
  return Qnil;
}

static VALUE
win_restorewindow(VALUE win)
{
  AnimXwinPtr wptr;

  Data_Get_Struct(win, AnimXwin, wptr);
  RestoreWindow(wptr);
  return Qnil;
}

static VALUE
win_redrawwindow(VALUE win)
{
  AnimXwinPtr wptr;

  Data_Get_Struct(win, AnimXwin, wptr);
  RedrawWindow(wptr);
  return Qnil;
}

static VALUE
win_readchar(VALUE obj)
{
  char c;

  c = ReadChar();
  return INT2NUM(c);
}

static VALUE
win_sensechar(VALUE obj)
{
  char c;

  c = SenseChar();
  return INT2NUM(c);
}

static VALUE
win_drawppm(VALUE win, VALUE img, VALUE x, VALUE y)
{
  AnimXwinPtr wptr;
  AnimXppmPtr pptr;

  Data_Get_Struct(win, AnimXwin, wptr);
  Data_Get_Struct(img, AnimXppm, pptr);
  DrawPPM(wptr, pptr, NUM2INT(x), NUM2INT(y));
  return Qnil;
}

static VALUE
win_getarea(VALUE win, VALUE img, VALUE x, VALUE y, VALUE width, VALUE height)
{
  AnimXwinPtr wptr;
  AnimXppmPtr pptr;

  Data_Get_Struct(win, AnimXwin, wptr);
  Data_Get_Struct(img, AnimXppm, pptr);
  GetArea(wptr, pptr, NUM2INT(x), NUM2INT(y), NUM2INT(width), NUM2INT(height));
  return Data_Wrap_Struct(cPPM, 0, 0, pptr);
}

static VALUE
win_setfont(VALUE obj, VALUE fontname)
{

  switch (TYPE(fontname)) {
  case T_STRING:
	SetFont(STR2CSTR(fontname));
	break;
  default:
	rb_raise(rb_eRuntimeError, "not valid value");
	break;
  }
  return Qnil;
}

static VALUE
win_setdrawmode(VALUE obj, VALUE mode)
{

  SetDrawMode(NUM2INT(mode));
  return Qnil;
}

static VALUE
win_setmapmode(VALUE obj, VALUE flag)
{

  SetMapMode(NUM2INT(flag));
  return Qnil;
}

static VALUE
win_autorepeat(VALUE obj, VALUE flag)
{
  int i;

  i = AutoRepeat(NUM2INT(flag));
  return INT2NUM(i);
}

static VALUE
win_readbutton(VALUE obj)
{
	int i;
	i = ReadButton();
	return INT2NUM(i);
}

static VALUE
win_sensebutton(VALUE obj)
{
	int i;
	i = SenseButton();
	return INT2NUM(i);
}

static VALUE
win_getmousepos(VALUE win)
{
  AnimXwinPtr wptr;
  int x, y;
  int i;
  VALUE ary;

  Data_Get_Struct(win, AnimXwin, wptr);
  i = GetMousePos(wptr, &x, &y);
  ary = rb_ary_new2(2);
  rb_ary_push(ary, INT2NUM(x));
  rb_ary_push(ary, INT2NUM(y));
  rb_ary_push(ary, INT2NUM(i));
  return ary;
}

static VALUE
win_flush(void)
{
  Flush();
}

static VALUE
ppm_s_new(VALUE class, VALUE filename, VALUE flag)
{
  AnimXppmPtr pptr;

  switch (TYPE(filename)) {
  case T_STRING:
	pptr = LoadPPM(STR2CSTR(filename), NUM2INT(flag));
	break;
  default:
	rb_raise(rb_eRuntimeError, "not valid value");
	break;
  }
  return Data_Wrap_Struct(cPPM, 0, 0, pptr);
}

static VALUE
ppm_geth(VALUE img)
{
  AnimXppmPtr pptr;

  Data_Get_Struct(img, AnimXppm, pptr);
  return INT2NUM(pptr->h);
}

static VALUE
ppm_getv(VALUE img)
{
  AnimXppmPtr pptr;

  Data_Get_Struct(img, AnimXppm, pptr);
  return INT2NUM(pptr->v);
}

void
Init_AnimX()
{
    mAnimX = rb_define_module("AnimX");
	cWin = rb_define_class_under(mAnimX, "Win", rb_cObject);
    rb_define_singleton_method(cWin, "new", win_s_new, 4);
    rb_define_method(cWin, "SetWindowTitle", win_setwindowtitle, 1);
    rb_define_method(cWin, "MoveWindow", win_movewindow, 2);
    rb_define_method(cWin, "ResizeWindow", win_resizewindow, 2);
    rb_define_method(cWin, "SetMaxSize", win_setmaxsize, 2);
    rb_define_method(cWin, "SetMinSize", win_setminsize, 2);
    rb_define_method(cWin, "LockCurrentSize", win_lockcurrentsize, 0);
    rb_define_method(cWin, "InstallColormap", win_installcolormap, 0);
    rb_define_method(cWin, "DrawPoint", win_drawpoint, 2);
    rb_define_method(cWin, "DrawLine", win_drawline, 4);
    rb_define_method(cWin, "DrawRect", win_drawrect, 4);
    rb_define_method(cWin, "DrawCirc", win_drawcirc, 4);
    rb_define_method(cWin, "FillRect", win_fillrect, 4);
    rb_define_method(cWin, "FillCirc", win_fillcirc, 4);
    rb_define_method(cWin, "DrawLines", win_drawlines, 1);
    rb_define_method(cWin, "FillLines", win_filllines, 1);
    rb_define_method(cWin, "DrawString", win_drawstring, 3);
    rb_define_method(cWin, "DrawKanji", win_drawkanji, 3);
    rb_define_method(cWin, "DrawImageString", win_drawimagestring, 3);
    rb_define_method(cWin, "DrawImageKanji", win_drawimagekanji, 3);
    rb_define_method(cWin, "DrawPPM", win_drawppm, 3);
    rb_define_method(cWin, "GetArea", win_getarea, 5);
    rb_define_method(cWin, "SaveWindow", win_savewindow, 0);
    rb_define_method(cWin, "RestoreWindow", win_restorewindow, 0);
    rb_define_method(cWin, "RedrawWindow", win_redrawwindow, 0);
    rb_define_method(cWin, "GetMousePos", win_getmousepos, 0);
    rb_define_method(cWin, "h", win_geth, 0);
    rb_define_method(cWin, "v", win_getv, 0);
	cPPM = rb_define_class_under(mAnimX, "PPM", rb_cObject);
    rb_define_singleton_method(cPPM, "new", ppm_s_new, 2);
    rb_define_method(cPPM, "h", ppm_geth, 0);
    rb_define_method(cPPM, "v", ppm_getv, 0);

    rb_define_module_function(mAnimX, "Flush", win_flush, 0);
    rb_define_module_function(mAnimX, "SetPenWidth", win_setpenwidth, 1);
    rb_define_module_function(mAnimX, "SetColor", win_setcolor, 1);
    rb_define_module_function(mAnimX, "SetBackColor", win_setbackcolor, 1);
    rb_define_module_function(mAnimX, "GetColor", win_getcolor, 1);
    rb_define_module_function(mAnimX, "GetRGB", win_getrgb, 3);
    rb_define_module_function(mAnimX, "SetFont", win_setfont, 1);
    rb_define_module_function(mAnimX, "SetDrawMode", win_setdrawmode, 1);
    rb_define_module_function(mAnimX, "SetMapMode", win_setmapmode, 1);
    rb_define_module_function(mAnimX, "LoadPPM", win_loadppm, 2);
    rb_define_module_function(mAnimX, "MakeEmptyPPM", win_makeemptyppm, 2);
    rb_define_module_function(mAnimX, "ReadChar", win_readchar, 0);
    rb_define_module_function(mAnimX, "SenseChar", win_sensechar, 0);
    rb_define_module_function(mAnimX, "AutoRepeat", win_autorepeat, 1);
    rb_define_module_function(mAnimX, "ReadButton", win_readbutton, 0);
    rb_define_module_function(mAnimX, "SenseButton", win_sensebutton, 0);

	rb_define_const(mAnimX, "Delay", INT2FIX(Delay));
	rb_define_const(mAnimX, "Direct", INT2FIX(Direct));
	rb_define_const(mAnimX, "Normal", INT2FIX(Normal));
	rb_define_const(mAnimX, "Force", INT2FIX(Force));
	rb_define_const(mAnimX, "AutoRepeatModeOff", INT2FIX(AutoRepeatModeOff));
	rb_define_const(mAnimX, "AutoRepeatModeOn", INT2FIX(AutoRepeatModeOn));
	rb_define_const(mAnimX, "AutoRepeatModeDefault", INT2FIX(AutoRepeatModeDefault));
	rb_define_const(mAnimX, "Button1", INT2FIX(Button1));
	rb_define_const(mAnimX, "Button2", INT2FIX(Button2));
	rb_define_const(mAnimX, "Button3", INT2FIX(Button3));

}
